@@ -1,5 +1,6 @@
 import random
 import numpy as np
+from ev3sim.simulation.randomisation import Randomiser
 
 
 class ColourSensorMixin:
@@ -14,9 +15,9 @@ class ColourSensorMixin:
 
     def _SenseValueAboutPosition(self, centrePosition, valueGetter):
         # Randomly sample value from SENSOR_POINTS chosen around the centrePosition.
-        points = [random.random() * self.SENSOR_RADIUS for _ in range(self.SENSOR_POINTS)]
+        points = [Randomiser.random() * self.SENSOR_RADIUS for _ in range(self.SENSOR_POINTS)]
         for x in range(len(points)):
-            angle = random.random() * 2 * np.pi
+            angle = Randomiser.random() * 2 * np.pi
             points[x] = valueGetter(np.array([np.cos(angle) * points[x], np.cos(angle) * points[x]]) + centrePosition)
             # For some reason the color module hangs otherwise :/
             if hasattr(points[x], "r"):
